#include "broadcast.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QDebug>

BroadCast::BroadCast(QIODevice* const iodevice, bool master ):  master_(master), iodevice_(iodevice)
{
	if(iodevice_ != nullptr) connect(iodevice_, &QIODevice::readyRead, this, &BroadCast::readyRead);
}

void BroadCast::write(const char * const buffer, const size_t length)
{
	QByteArray mBuffer("bcst: ");
	for (size_t i = 0; i < length; ++i)
	{
		if(buffer[i] != '\n' && buffer[i] != '\0') mBuffer.push_back(buffer[i]);
		else
		{
			mBuffer.push_back('\\');
			if(buffer[i] == '\n')mBuffer.push_back('n');
			else mBuffer.push_back('0');
		}
	}
	mBuffer.push_back('\n');
	if(iodevice_)iodevice_->write(mBuffer);
}

void BroadCast::write(const QByteArray& buffer)
{
	write(buffer.data(), buffer.size());
}

void BroadCast::sendJson(const QJsonObject& json)
{
	QJsonDocument jsonDocument(json);
	QByteArray buffer("JSON: ");
	buffer.append(jsonDocument.toJson());
	write(buffer);
}

void BroadCast::sendSensors()
{
	if(iodevice_)for(auto& sensor: *globalSensors.getSensors())
		{
			iodevice_->write("bcst: "+sensor.toString().toLatin1()+'\n');
		}
}

void BroadCast::requestSensors()
{
	if(iodevice_)iodevice_->write("bcst: GETSENSORS\n");
}

void BroadCast::requestJson()
{
	if(iodevice_)iodevice_->write("bcst: GETJSN\n");
}

void BroadCast::decodeMaster(const QByteArray& buffer)
{
	if(buffer.startsWith("GETJSN"))
	{
		qDebug()<<"json requested";
		jsonRequested();
	}
	else if(buffer.startsWith("GETSENSORS") )
	{
		qDebug()<<"sensors requested";
		sendSensors();
	}
}

void BroadCast::decode(QByteArray buffer)
{
	qDebug()<<"decodeing: "<<buffer;
	if(buffer.size() >= 6 && buffer[0] == 'J' && buffer[1] == 'S' && buffer[2] == 'O' && buffer[3] == 'N'
	        && buffer[4] == ':')
	{
		qDebug()<<"got json";
		buffer.remove(0,6);
		for(int i = 0; i < buffer.size()-1; ++i)
		{
			if( buffer[i] == '\\' && buffer[i+1] == 'n' )
			{
				buffer[i] = '\n';
				buffer.remove(i+1,1);
			}
			else if( buffer[i] == '\\' && buffer[i+1] == '0' )
			{
				buffer[i] = '\0';
				buffer.remove(i+1,1);
			}
		}

		QJsonParseError error;
		QJsonDocument document = QJsonDocument::fromJson(buffer, &error);

		qDebug()<<"JSON:";
		qDebug()<<document.toJson().data();
		QJsonObject jsonObject = document.object();
		if(error.error == QJsonParseError::NoError && !document.isEmpty() && !jsonObject.isEmpty()) gotJson(jsonObject);
		else
		{
			qDebug()<<error.errorString();
		}
	}
	else if(buffer.size() >= 6 && buffer[0] == 'S' && buffer[1] == 'E' && buffer[2] == 'N' && buffer[3] == 'S'
	        && buffer[4] == 'O' && buffer[5] == 'R')
	{
		Sensor sensor = Sensor::sensorFromString(buffer);
		if(sensor.type != Sensor::TYPE_DUMMY) gotSensorState(sensor);
	}
}

void BroadCast::sendMessage(const QString &title, const QString &body)
{
	write(QByteArray("MESG ") + title.toLatin1() + " BODY " + body.toLatin1());
}

void BroadCast::readyRead()
{
	buffer_.append(iodevice_->readAll());
	int newlineIndex = buffer_.indexOf('\n');
	while( newlineIndex != -1 )
	{
		if(buffer_.startsWith("bcst: "))
		{
			QByteArray tmp = buffer_.mid(6,newlineIndex-6);
			decode(tmp);
			if(master_)decodeMaster(tmp);
		}
		buffer_.remove(0, newlineIndex+1);
		newlineIndex = buffer_.indexOf('\n');
	}
}
